import os
import shutil
from pathlib import Path

import pytest

from ktdk import KTDK, scenarios
from tests.paths import TEST_RESOURCES_BASE


def create_cmake(src, *names):
    content = """
    cmake_minimum_required (VERSION 3.0)
    project (kontr_tests)
    
    add_compile_options(-std=c++11)
    """
    src = Path(src)
    for name in names:
        content += f"add_executable({name} {name}.cpp)\n"
    cmake_file = src.joinpath('CMakeLists.txt')
    cmake_file.write_text(content)


@pytest.fixture
def workspace_dir(tmpdir):
    workspace = tmpdir.mkdir('workspace')
    return workspace


@pytest.fixture
def test_files_dir(tmpdir):
    test_files = tmpdir.mkdir('test_files')
    return test_files


@pytest.fixture
def submission_dir(tmpdir):
    submission = tmpdir.mkdir('submission')
    return submission


@pytest.fixture(autouse=True)
def prepared_sources(workspace_dir):
    path = os.path.join(workspace_dir, 'src')
    shutil.copytree(str(TEST_RESOURCES_BASE / 'cpp' / 'checkstyle'), path)
    create_cmake(path, 'correct', 'fail_format', 'fail_tidy')
    return workspace_dir


@pytest.fixture()
def scenario(prepared_sources):
    return scenarios.StyleCheckScenario(checkstyle_points=3, tidy_points=1,
                                        files_glob="src/*.cpp", headers=["src/meta.h"])


@pytest.fixture()
def ktdk_instance(test_files_dir, workspace_dir, submission_dir):
    return KTDK(
        test_files=test_files_dir,
        submission=submission_dir,
        workspace=workspace_dir,
        devel=True
    )


@pytest.fixture()
def ktdk(scenario, ktdk_instance):
    ktdk_instance.suite.add_task(scenario)
    return ktdk_instance


def test_checkstyle_scenario(ktdk):
    ktdk.invoke()
    assert ktdk.suite.result.effective.failed
    assert ktdk.stats['final_points'] == 1



"""
Logging configuration module
"""
import logging
from logging.config import dictConfig

FORMATTERS = {
    'verbose': {
        'format': '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
    },

    'simple': {
        'format': '%(levelname)s %(message)s'
    },
    'colored_console': {
        '()': 'coloredlogs.ColoredFormatter',
        'format': "%(asctime)s - %(name)s - %(levelname)s - %(message)s", 'datefmt': '%H:%M:%S'
    },
}

HANDLERS = {
    'console': {
        'level': 'DEBUG',
        'class': 'logging.StreamHandler',
        'formatter': 'colored_console'
    },
}

LOGGERS = {
    'ktdk': {'handlers': ['console'], 'level': 'TRACE', 'propagate': True},
    'tests': {'handlers': ['console'], 'level': 'DEBUG', 'propagate': True},
}

LOGGING_CONF = {
    'version': 1,
    'disable_existing_loggers': True,
    'formatters': FORMATTERS,
    'handlers': HANDLERS,
    'loggers': LOGGERS,
}

TRACE_LOG_LVL = 9


def _trace(self, message, *args, **kws):
    if self.isEnabledFor(TRACE_LOG_LVL):
        # Yes, logger takes its '*args' as 'args'.
        self._log(TRACE_LOG_LVL, message, args, **kws)


def add_custom_log_level():
    logging.addLevelName(TRACE_LOG_LVL, 'TRACE')
    logging.Logger.trace = _trace


def load_config():
    """Loads config based on the config type
    """
    add_custom_log_level()
    dictConfig(LOGGING_CONF)



"""
Main KTDK module
"""
import json
import logging
from typing import Callable, List

import requests

from ktdk import log_config
from ktdk.core import Result, Task, Test
from ktdk.runtime import Context
from ktdk.runtime.stat import stat_test
from ktdk.utils import unique_name

log_config.load_config()

log = logging.getLogger(__name__)

DEFAULTS_CONFIG = {
    'test_files': 'test_files',
    'workspace': 'workspace',
    'submission': 'submission',
    'results': None,
    'timeout': 60,  # seconds
    'suite_timeout': 3600,  # seconds
    'tags': [],
    'webhook_url': None,
    'webhook_token': None,
    'safe_suffixes': False,
    'cpp_compiler': 'clang++',
    'cpp_compiler_flags': '-std=c++17 -Wall -Wextra -pedantic',
    'c_compiler': 'clang',
    'c_compiler_flags': '-std=c99 -Wall -Wextra -pedantic',
    'valgrind_penalization': 0.7,
    'dump_result': False,
    'clang_format_style': 'pb161',
    'clang_format_styles_dir': '/tmp/check_style'
}


def dump_to_file(context: Context, file_name: str, what):
    """Dumps content to the file
    Args:
        context(Context): Context instance
        file_name(str): File name
        what: What should be written to the file
    """
    file_name = unique_name(file_name, context=context) + ".json"
    context.paths.save_result(file_name, json.dumps(what))


class KTDK(object):
    """
    Main class of the KTDK, it holds configuration, the suite (root of the )
    """
    instance = None

    @staticmethod
    def get_instance(**config):
        if KTDK.instance is None:
            log.debug(f"[KTDK] Instance config: {config}")
            KTDK.instance = KTDK(**config)
        return KTDK.instance

    def __init__(self, **config):
        """Creates instance of the KTDK suite
        Args:
            **config:
        """
        self.suite = Test(name='suite', tags=['suite'])
        self._post_actions: List[Callable] = []
        defaults = self.__config_defaults()
        self.config = {**defaults, **config}
        if self.config.get('devel'):
            for key, val in self.config.items():
                log.debug(f"[KTDK] Config: {key}={val}")

    def register_tags(self, *tags):
        if 'registered_tags' not in self.config:
            self.config['registered_tags'] = []
        self.config['registered_tags'].extend(tags)

    @property
    def post_actions(self) -> List[Callable]:
        """Gets all the post actions
        Returns(List[Callable]): List of all post actions
        """
        return self._post_actions

    def add_post_action(self, post_action: Callable):
        """Adds post action - action that will be executed after suite has been executed
        Args:
            post_action(Callable): Function that takes KTDK instance as the 1st arg and
                                    the Context as the 2nd arg

        """
        self._post_actions.append(post_action)

    def post_action(self):
        """Decorator that defines the post action
        Returns(Callable): Decorated Callable
        """

        def __post_action(func):
            self.add_post_action(func)

        return __post_action

    def create_context(self) -> Context:
        """Creates context from the configuration
        Returns(Context): The context instance
        """
        context = Context(suite_config=self.config, test_config={})

        return context

    def invoke(self) -> Result:
        """Invokes the KTDK suite
        Returns(Result): Overall result of the suite
        """
        return self.run()

    def run(self) -> Result:
        """Runs whole KTDK suite adn saves result

        Returns(Result): Overall result of the suite
        """
        context = self.create_context()
        log.info(f"[CTX] Created: {context}")
        suite_runner = self.suite.runner.get_instance(context=context)
        suite_result = suite_runner.invoke()
        self.__dump_required_files(context)
        self.__invoke_post_actions(context)
        return suite_result

    @property
    def stats(self) -> dict:
        return stat_test(self.suite)

    @staticmethod
    def __config_defaults():
        return DEFAULTS_CONFIG

    def __invoke_post_actions(self, context: Context):
        for post_action in self.post_actions:
            post_action(self, context)

        self.__send_notification(context=context)

    def __dump_required_files(self, context):
        if context.config['dump_result']:
            dump_to_file(context, 'suite-result', self.suite.to_dict())
        dump_to_file(context, 'suite-stats', self.stats)
        dump_to_file(context, 'suite-files', context.config['work_files'])

    def __send_notification(self, context: Context):
        webhook_url = context.config['webhook_url']
        if webhook_url:
            params = dict(url=webhook_url, data=json.dumps(self.stats))
            if context.config['webhook_token']:
                params['headers'] = {'Authorization': 'Bearer ' + context.config['webhook_token']}
            requests.post(**params)



import logging
import subprocess
from pathlib import Path

from ktdk.utils import naming
from ktdk.utils.basic import BasicObject
from ktdk.utils.naming import unique_suffix

log = logging.getLogger(__name__)


class Command(object):
    def __init__(self, command, args: list = None, output_name=None, task=None, **config):
        """ Creates instance of the command
        Args:
            command(str): Command string
            args(list): Command arguments
            **config: Configuration dictionary
        """
        self.command = str(command)
        self._args = args if args else []
        self.config = {**self.__default_config, **config}
        self._output_name = output_name
        self._unique_suffix = None
        self._task = None

    @property
    def unique_suffix_for_command(self):
        if self._unique_suffix is None:
            self._unique_suffix = unique_suffix(context=self.context)
        return self._unique_suffix

    def output_file_name(self, suffix, name=None):
        """Name with the unique suffix
        Args:
            suffix(str): Suffix
            name(str): Name of the file (default is extracted from command)
        Returns(str): Full name with unique name
        """
        name = name or Path(self.command).name
        if self._output_name:
            name = f"{name}::{self._output_name}"
        name = naming.slugify(name)
        if self.unique_suffix_for_command is not None:
            name = f"{name}_{self.unique_suffix_for_command}"
        if self.task and self.task.test:
            name = self.task.test.namespace + "::" + name
        return name + "." + suffix

    def set_task(self, task):
        """Injects context
        Args:
            task(Task): Context
        """
        self._task = task
        return self

    @property
    def context(self):
        """Gets Context instance
        Returns(Context): Context instance
        """
        if self._task is None:
            return None
        return self._task.context

    @property
    def task(self):
        return self._task

    @property
    def results_path(self):
        """Results path (None if not context)
        Returns(Path): the path to the results
        """
        if not self.context:
            return None
        return self.context.paths.results

    def timeout(self, timeout):
        """Sets timeout
        Args:
            timeout(int): Timeout
        Returns(Command): Command instance
        """
        self.config['timeout'] = timeout
        return self

    def stdin(self, input_stream):
        """Sets stdin
        Args:
            input_stream: Input stream
        Returns(Command): Command instance
        """
        self.config['stdin'] = input_stream
        return self

    def shell(self, shell: bool):
        """Sets whether to use shell
        Args:
            shell(bool): Shell
        Returns(Command): Command instance
        """
        self.config['shell'] = shell
        return self

    def cwd(self, current_working_dir):
        """Sets current working directory
        Args:
            current_working_dir(path): Current working directory path
        Returns(Command): Command instance
        """
        self.config['cwd'] = current_working_dir
        return self

    def check(self, check: bool):
        """Whether to check the command return core
        Args:
            check(bool): Check the command return core
        Returns(Command): Command instance
        """
        self.config['check'] = check
        return self

    def add_args(self, *args):
        """Adds arguments to command
        Args:
            *args: Command arguments
        Returns(Command): Command instance
        """
        self._args.extend(args)
        return self

    def stdout(self, out_stream):
        """Sets the stdout
        Args:
            out_stream: STD Output stream
        Returns(Command): Command instance
        """
        self.config['stdout'] = out_stream
        return self

    def stderr(self, out_stream):
        """Sets the stderr
        Args:
            out_stream: ERR Output stream
        Returns(Command): Command instance
        """
        self.config['stderr'] = out_stream
        return self

    @property
    def args(self):
        """Gets list of the arguments
        Returns(list): List of the arguments
        """
        return self._args

    @property
    def full_command(self):
        """Gets full command with arguments
        Returns(list): Command with arguments
        """
        full = [self.command]
        full.extend([*self.args])
        return full

    @property
    def __default_config(self):
        """Gets default config
        Returns(dict): Returns the dictionary
        """
        return {
            "stdout": subprocess.PIPE,
            "stderr": subprocess.PIPE,
            "shell": False,
            "cwd": None,
            "timeout": None,
            "check": False,
            "encoding": None,
            "errors": None,
        }

    def invoke(self, **kwargs):
        """Invokes the command
        Returns: Command result
        """
        return self.run(**kwargs)

    def execute(self, **kwargs):
        """Executes the command
         Returns: Command result
         """
        config = {**self.config}
        config.update(**kwargs)
        log.debug(f"[EXE] Command: \"{self.full_command}\" with config: ({config})")
        log.debug(f"[CMD_EXEC] Exec: {' '.join(self.full_command)}")
        try:
            if config['timeout'] is None:
                config['timeout'] = self.context.config['timeout'] if self.context else 60
            return subprocess.run(self.full_command, **config)
        except subprocess.TimeoutExpired as ex:
            log.warning(f"{ex}")
            return ex

    def process(self, result):
        """Process the command
        Args:
            result: Command result
        Returns(CommandResult): Returns the custom command result wrapper
        """
        return CommandResultProcessor(result=result, command=self).process()

    def run(self, **kwargs):
        result = self.execute(**kwargs)
        log.debug(f"[EXE] Result: \"{self.command}\": {result}")
        return self.process(result)


class CommandResultProcessor(object):
    def __init__(self, command, result):
        """Creates instance of the command result processor
        Args:
            command(Command): Command
            result: Command execution result
        """
        self.command = command
        self.result = result
        self.outputs = {}

    @property
    def context(self):
        """Gets instance of the context
        Returns(Context): Instance of the context
        """
        return self.command.context

    def save_buffer(self, buffer, suffix):
        """Saves the buffer to the file
        Args:
            buffer(str): Buffer to be saved
            suffix(str): File path suffix
        Returns(str): filename
        """
        if not self.context:
            return None

        file_name = self.command.output_file_name(suffix)
        full_path = Path("outputs") / file_name
        path = self.context.paths.save_result(full_path, content=buffer)
        return path

    def __save_outputs(self):
        def __save_output(which, buffer):
            self.outputs[which] = self.save_buffer(buffer, suffix=which)

        __save_output('stdout', self.result.stdout.decode('utf-8'))
        __save_output('stderr', self.result.stderr.decode('utf-8'))

    def process(self):
        """Processes the command result
        Returns(CommandResult): Command result instance
        """
        self.__save_outputs()
        return CommandResult(self.result, self.outputs)


class CommandBytesBuffer(object):
    def __init__(self, str_bytes, path=None):
        """Creates Bytes buffer
        Args:
            str_bytes(bytes): Bytes
        """
        self.path = path
        self.bytes = str_bytes

    @property
    def empty(self) -> bool:
        """Whether the output is empty
        Returns(bool): true if the buffer is empty
        """
        return not self.bytes

    @property
    def content(self):
        """String content
        Returns(str): Buffer content represented by string
        """
        return self.bytes.decode('utf-8')

    def __call__(self):
        """Returns the bytes output
        Returns(bytes): Output buffer
        """
        return self.bytes

    def __str__(self):
        """Returns the output buffer as string
        Returns(str): Output string
        """
        return self.content


class CommandResult(BasicObject):
    def __init__(self, result, outputs):
        """Initializes Command result
        Args:
            result: Command execution result
            outputs(dict): Dictionary with generated outputs
        """
        super().__init__()
        self.result = result
        self.outputs = outputs

    @property
    def stdout(self):
        """ Stdout buffer
        Returns (CommandBytesBuffer): Command bytes buffer
        """
        return CommandBytesBuffer(self.result.stdout, self.outputs.get('stdout'))

    @property
    def stderr(self):
        """Stderr buffer
        Returns (CommandBytesBuffer): Command bytes buffer
        """
        return CommandBytesBuffer(self.result.stderr, self.outputs.get('stderr'))

    @property
    def return_code(self):
        """Return core of the
        Returns (int): Status core of the program execution
        """
        if hasattr(self.result, "returncode"):
            return self.result.returncode
        return 1

    @property
    def ok(self):
        """Program existed OK
        Returns(bool): True if return core is zero or it didn't timeout
        """
        return not self.timeout and self.return_code == 0

    @property
    def nok(self):
        """Program does not existed with OK
        Returns(bool): Unless OK
        """
        return not self.ok

    @property
    def timeout(self):
        """Program timed out
        Returns(bool): True if timeout
        """
        if hasattr(self.result, "timeout"):
            return self.result.timeout
        return False



from ktdk.asserts.utils import get_context_diff


class GeneralMatcher(object):
    def __init__(self, message=None, symbol=None):
        self.message = message
        self._symbol = symbol or '(+)'

    def __call__(self, orig):
        return self.invoke(orig)

    @property
    def symbol(self):
        return self._symbol

    # pylint: disable=no-self-use
    def message_footer(self, orig):
        return f"Provided: {orig}"

    def predicate(self, orig):
        return orig is not None

    def get_object(self, orig):
        return orig

    # pylint: enable=no-self-use

    def generate_message(self, orig):
        full_message = ""
        if self.message:
            full_message += str(self.message) + "\n"
        full_message += self.message_footer(orig) + "\n"
        return full_message

    def invoke(self, orig):
        cond = self.predicate(orig)
        message = self.generate_message(orig)
        obj = self.get_object(orig)
        return cond, obj, message


class GeneralExpectedMatcher(GeneralMatcher):
    def __init__(self, expected, message=None, use_diff=False, symbol=None):
        super().__init__(message=message, symbol=symbol)
        self._use_diff = use_diff
        self.expected = expected

    def obj_message(self, orig):
        return f"{orig} {self.symbol} {self.expected}"

    def generate_message(self, orig):
        return f"{self.obj_message(orig)}\n{self.message_footer(orig)}"

    def message_footer(self, orig):
        if self._use_diff:
            return get_context_diff(expected=self.expected, provided=orig)
        return f"Expected: {self.expected}\nProvided: {orig}"

    def predicate(self, orig):
        return orig == self.expected



import os
import shutil
import subprocess

import pytest

from ktdk import Test
from ktdk.tasks.cpp.catch import CatchRunTestsOneByOneTask
from ktdk.utils.flatters import flatten_tests
from tests.paths import TEST_RESOURCES_BASE
from tests.utils import get_test_context


def prepare_and_build(path):
    old_dir = os.getcwd()
    os.chdir(path)
    subprocess.run(['clang++', '-std=c++14', '-o', 'tests', 'tests.cpp'])
    subprocess.run(['clang++', '-std=c++14', '-o', 'fails', 'fails.cpp'])
    os.chdir(old_dir)
    return path


@pytest.fixture
def prepared_dir(tmpdir):
    path = tmpdir.mkdir('workspace')
    path = os.path.join(path, 'build')
    shutil.copytree(str(TEST_RESOURCES_BASE / 'catch'), path)
    return prepare_and_build(path)


@pytest.fixture
def context(prepared_dir):
    config = dict(workspace=prepared_dir)
    pass_path = os.path.join(prepared_dir, 'tests')
    fail_path = os.path.join(prepared_dir, 'fails')
    test_config = dict(exec={'tests': pass_path, 'fails': fail_path})
    return get_test_context(suite_config=config, test_config=test_config)


@pytest.fixture()
def task(context):
    return CatchRunTestsOneByOneTask(executable='tests')


@pytest.fixture()
def fail_task(context):
    return CatchRunTestsOneByOneTask(executable='fails')


@pytest.fixture()
def test(context, task) -> Test:
    test = Test(name='test_task')
    test.add_task(task)
    return test


@pytest.fixture()
def fail_test(context, fail_task) -> Test:
    test = Test(name='test_task')
    test.add_task(fail_task)
    return test


def test_catch_one_by_one_task(context, test, task):
    runner = test.runner.get_instance(context=context)
    runner.invoke()
    assert test.result.effective.passed
    assert test.tests
    for t in flatten_tests(test, include_self=False):
        test_suite = t.context.config['xunit_suite']
        assert test_suite
        assert test_suite.errors == 0
        assert test_suite.failures == 0


def test_catch_failed_task(context, fail_task, fail_test):
    test = fail_test

    runner = test.runner.get_instance(context=context)
    runner.invoke()
    assert test.result.effective.failed
    assert test.tests



from ktdk.core import results
from ktdk.core.results import Result


def test_construct_result():
    res = results.NONE
    assert res.state == Result.NONE
    assert res.nok
    assert not res.ok
    assert res.reason == 'NONE'


def test_pass_result():
    res = results.PASS
    assert res.ok
    assert not res.nok
    assert res.passed
    assert not res.errored
    assert not res.failed
    assert not res.skipped
    assert res.reason == 'PASS'


def test_fail_result():
    res = results.FAIL
    assert not res.ok
    assert res.nok
    assert not res.passed
    assert not res.errored
    assert res.failed
    assert not res.skipped
    assert res.reason == 'FAIL'


def test_skip_result():
    res = results.SKIP
    assert res.ok
    assert not res.nok
    assert not res.passed
    assert not res.errored
    assert not res.failed
    assert res.skipped
    assert res.reason == 'SKIP'


def test_error_result():
    res = results.ERROR
    assert not res.ok
    assert res.nok
    assert not res.passed
    assert res.errored
    assert not res.failed
    assert not res.skipped
    assert res.reason == 'ERROR'



from ktdk.tasks.cpp.cmake import CMakeBuildTask
from tests.utils import get_test_context


def prepare_dir(tmpdir):
    path = tmpdir.mkdir('workspace')
    src = path.mkdir('src')
    c_file = src.join('main.c')
    c_file.write("""
        #include <stdio.h>\n
        int main() { printf("ahoj svet!\\n"); return 0; }
    """)
    cmake_file = src.join('CMakeLists.txt')
    cmake_file.write("""
        add_executable(hello main.c)
    """)
    return path


def create_context(tmpdir):
    config = dict(workspace=prepare_dir(tmpdir))
    return get_test_context(suite_config=config)


def test_cmake_tasks(tmpdir):
    cmake = CMakeBuildTask(source='src')
    context = create_context(tmpdir=tmpdir)
    runner = cmake.runner.get_instance(context=context)
    runner.invoke()
    assert cmake.result.effective.passed
    assert context.config['exec']



import sys

import importlib
import importlib.machinery
import importlib.util
import logging
import os
from pathlib import Path

from ktdk import KTDK

log = logging.getLogger(__name__)


class CliManager(object):
    def __init__(self):
        self.config = load_vars()

    @property
    def workspace_dir(self) -> Path:
        return Path(self.config['workspace_dir']).absolute()

    @property
    def test_files_dir(self) -> Path:
        return Path(self.config['test_files_dir']).absolute()

    @property
    def submission_dir(self) -> Path:
        return Path(self.config['submission_dir']).absolute()

    @property
    def results_dir(self) -> Path:
        return Path(self.config['results_dir']).absolute()

    def load_project_files(self):
        module_path = self.test_files_dir / 'kontr_tests'
        sys.path.insert(0, str(module_path))
        path = module_path / self.config.get('entry_point')
        return _import_module_files('test_files', path)

    def get_ktdk(self, **kwargs):
        config = {**self.config, **kwargs}
        ktdk = KTDK.get_instance(**config)
        return ktdk

    def create_context(self, **kwargs):
        ktdk = self.get_ktdk(**kwargs)
        self.load_project_files()
        return ktdk


def _load_var(dictionary: dict, name: str, default=None) -> dict:
    dictionary[name] = os.getenv(f'KTDK_{name.upper()}', default)
    return dictionary


def load_vars() -> dict:
    dictionary = {}
    _load_var(dictionary, 'workspace_dir', '/tmp/workspace')
    _load_var(dictionary, 'test_files_dir', '/tmp/test_files')
    _load_var(dictionary, 'submission_dir', '/tmp/submission')
    _load_var(dictionary, 'results_dir', '/tmp/results')
    _load_var(dictionary, 'entry_point', 'instructions.py')
    _load_var(dictionary, 'webhook_url')
    _load_var(dictionary, 'webhook_token')
    _load_var(dictionary, 'test_timeout', 60)
    _load_var(dictionary, 'suite_timeout', 3600)
    _load_var(dictionary, 'suite_id')
    return dictionary


def _import_module_files(module_name: str, path: Path):
    full_path = str(path)
    loader = importlib.machinery.SourceFileLoader(module_name, full_path)
    spec = importlib.util.spec_from_loader(loader.name, loader)
    mod = importlib.util.module_from_spec(spec)
    loader.exec_module(mod)
    log.info(f"[CLI] Loading module: {mod}")
    return mod



from ktdk.asserts.matchers import IsNotNone, Equals
from ktdk.core.tasks import Task
from ktdk.core.tests import Test
from ktdk.runtime.context import Context


class SimpleCounterLocalTask(Task):
    def _run(self, *args, **kwargs):
        self.context.config.set_test('lc', 1)
        self.asserts.check(self.context.config, IsNotNone())


class SimpleLocalLevelTask(Task):
    def _run(self, *args, **kwargs):
        self.asserts.check(self.context.config['lc'], Equals(1))
        self.context.config.set_test('lc', 2)
        self.asserts.check(self.context.config, IsNotNone())


class SimpleCounterGlobalTask(Task):
    def _run(self, *args, **kwargs):
        self.context.config.set_suite('gc', 1)
        self.asserts.check(self.context.config, IsNotNone())


class FailingTask(Task):
    def _run(self, *args, **kwargs):
        self.asserts.check(None, IsNotNone())


def test_test_runner_with_one_test():
    test = Test(name='test_name')
    runner = test.runner.get_instance(context=Context())
    runner.invoke()
    assert test.result.effective.passed


def test_test_runner_with_two_tests():
    test1 = Test(name='first')
    test2 = Test(name='second')
    test1.add_test(test2)
    runner = test1.runner.get_instance(context=Context())
    runner.invoke()
    assert test1.result.effective.passed


def test_test_runner_with_two_tests_and_task():
    test1 = Test(name='first')
    test2 = Test(name='second')
    task = SimpleCounterLocalTask('count task')
    test1.add_test(test2)
    test2.add_task(task)
    context = Context(suite_config={'gc': 0}, test_config={'lc': 0})
    runner = test1.runner.get_instance(context=context)
    result = runner.invoke()
    assert result
    assert 0 == context.config['lc']


def test_test_runner_with_two_tests_and_global_task():
    test1 = Test(name='first')
    test2 = Test(name='second')
    task = SimpleCounterGlobalTask('count task')
    test1.add_test(test2)
    test2.add_task(task)
    context = Context(suite_config={'gc': 0}, test_config={'lc': 0})
    runner = test1.runner.get_instance(context=context)
    result = runner.invoke()
    assert result
    assert 1 == context.config['gc']


def test_test_runner_with_two_tests_and_failing_task():
    test1 = Test(name='first')
    test2 = Test(name='second')
    task = FailingTask('count task')
    test1.add_test(test2)
    test2.add_task(task)
    context = Context(suite_config={'gc': 0}, test_config={'lc': 0})
    runner = test1.runner.get_instance(context=context)
    result = runner.invoke()
    assert result


def test_test_runner_with_two_tests_and_two_tasks():
    test1 = Test(name='first')
    test2 = Test(name='second')
    task = SimpleCounterLocalTask('count task')
    task2 = SimpleLocalLevelTask('count task2')
    test1.add_test(test2)
    test2.add_task(task)
    test2.add_task(task2)
    context = Context(suite_config={'gc': 0}, test_config={'lc': 0})
    runner = test1.runner.get_instance(context=context)
    result = runner.invoke()
    assert result
    assert 0 == context.config['lc']
